import builtins
import typing

from gi.repository import GObject


class Display():
    ...


class Screen():
    ...


class Visual():
    ...


class XConfigureEvent():
    ...


class XFontStruct():
    ...


class XImage():
    ...


class XTrapezoid():
    ...


class XVisualInfo():
    ...


class XWindowAttributes():
    ...


class XEvent():
    ...


def open_display() -> None: ...


